;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; POW Block (Mario-Bros)
;; By Sonikku
;; Description: A sprite that walks hangs suspended in the air. If hit from below, it will stun, damage, or defeat
;; all enemies on the ground.
;; 
;; This sprite uses custom interaction with most forms of damage, barring sprites.
;; I have documented which CFG editor features are enabled through additional code.
;; 
;; Supported Features:
;;;; Extra Bit:
;;;;;; - If set, the sprite will align itself a half tile (8 pixels) to the right.
;;;;;; 
;;;; Extra Property Byte 1: CSFJxxxx ********************************************************* Bit value
;;;;;; - x: unused ------------------------------------------------------------------------------ $01
;;;;;; - x: unused ------------------------------------------------------------------------------ $02
;;;;;; - x: unused ------------------------------------------------------------------------------ $04
;;;;;; - x: unused ------------------------------------------------------------------------------ $08
;;;;;; - J: Can be damaged/triggered by spin jumps. --------------------------------------------- $10
;;;;;; - F: Can be damaged/triggered by fireballs. ---------------------------------------------- $20
;;;;;; - S: Can be damaged/triggered by the cape. ----------------------------------------------- $40
;;;;;; - C: Carriable. Can be picked up by standing on it and pressing the button. -------------- $80
;;;; Default CFG Files:
;;;; - PowBlock.cfg:			Can be triggered by fire, cape, spin jumping, or hitting from below and can be carried.
;;;; - PowBlock_Uncarriable.cfg:	Same as above but cannot be carried.

!SpinyNum =		$00		; sprite number of the Mario Bros Spiny.		; \
!FighterFlyNum =	$01		; sprite number of the Mario Bros Fighter Fly.		;  | change the $FF to the corresponding values of your sprites.txt; leave at $FF if not using.
!SideStepperNum =	$02		; sprite number of the Mario Bros Side Stepper.		; /

!NumToWhitelist =	$02		; number of sprites to whitelist

SpriteTable:
	db !SpinyNum,$01,$01		; Custom Sprite: Spiny; Damage Type: "Mario Bros"
	db !FighterFlyNum,$01,$01	; Custom Sprite: Fighter Fly; Damage Type: "Mario Bros"
	db !SideStepperNum,$01,$01	; Custom Sprite: Side Stepper; Damage Type: "Mario Bros"
;; Most sprites should behave properly with the default "cape smash"/"bounce block" damage. In the event that they do
;; not, you can add entries to the whitelist and program in what you feel should happen to the sprite(s) when this
;; sprite is triggered.
;; To find where to add more pointers to point to custom code to execute as a damage type, search this file for BOOKMARK1.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	print "INIT ",pc
	LDA $7FAB10,x	; \
	AND #$04	;  | branch if extra bit not set
	BEQ +		; /
	REP #$20	; \
	LDA #$0008	;  | 
	SEP #$20	;  | 
	CLC		;  | 
	ADC $E4,x	;  | move sprite over a bit
	STA $E4,x	;  | 
	XBA		;  | 
	ADC $14E0,x	;  | 
	STA $14E0,x	; /
	STZ $1510,x	; 
+	RTL		; 

	print "MAIN ",pc
	PHB
	PHK
	PLB
	JSR PowBlockMain
	PLB
	RTL

PowBlockMain:
	JSR SUB_OFF_SCREEN_X0
+	JSR SubGfx	; load graphics
	LDA $9D		; \ return if sprites locked
	BNE .return	; /
	LDA $14C8,x	; \
	CMP #$08	;  | return if sprite isn't "normal"
	BCC .return	; /
	BEQ .normal	; 
	CMP #$0B	; \ branch if carried
	BEQ .return	; /
	JSL $01A7DC	; default interaction
	LDA $1588,x	; \
	AND #$04	;  | branch if in the air
	BEQ .return	; /
	LDA #$01	; \ play sound
	STA $1DF9	; /
	JSR SubSmoke	; 
	LDA $1570,x	; \
	ASL : ASL	;  | set timer based on current POW state
	EOR #$0C	;  | 
	STA $1558,x	; /
	INC $1510,x	; mark sprite as dying
	LDA #$08	; \ set regular state
	STA $14C8,x	; /
	JSR DamageSprites
	RTS
.normal	LDA $1510,x	; \ branch if not marked as dying
	BEQ .notdying	; /
	LDA $1558,x	; \ animate if timer is nonzero
	BNE +		; /
	STZ $14C8,x	; kill sprite
+	LSR : LSR	; \
	TAY		;  | animate
	LDA .frame,y	;  | 
	STA $151C,x	; /
	RTS		; 
.frame	db $02,$01,$00
.notdying
	JSR CapeContact
	JSR MarioContact
	JSR FireballContact
	LDA $1570,x	; \ current state = current frame
	STA $151C,x	; /
	LDA $163E,x	; \
	CMP #$06	;  | branch if not time to increase
	BNE .return	; /
	INC $1570,x	; increase times used
	LDA $1570,x	; \
	CMP #$03	;  | branch if not used 3 times
	BCC .return	; /
	STZ $14C8,x	; kill sprite
	JSL $07FC3B	; show stars
.return	RTS		; 

CapeContact:
	LDA $7FAB28,x	; \ 
	AND #$40	;  | 
	BEQ .no_contact	; /
	LDA $166E,x	; \
	AND #$20	;  | 
	ORA $1FE2,x	;  | 
	BNE .no_contact	; /
	LDA $1632,x	; \
	PHY		;  | 
	LDY $74		;  | 
	BEQ +		;  | don't interact with cape if mario and the sprite
	EOR #$01	;  | are not both above/behind the foreground together
+	PLY		;  | 
	EOR $13F9	;  | 
	BNE .no_contact	; /
	LDA $13E8	; \ don't interact with cape if cape contact is disabled
	BEQ .no_contact	; /
	LDA $15D0,x	; \
	ORA $1FE2,x	;  | don't interact with cape if sprite is being eaten or it just shouldn't
	BNE .no_contact ; /
	LDA $13E9	; \
	SEC		;  | 
	SBC #$02	;  | 
	STA $00		;  | set cape clipping x position
	LDA $13EA	;  | 
	SBC #$00	;  | 
	STA $08		; /
	LDA #$14	; \ set cape clipping width
	STA $02		; /
	LDA $13EB	; \
	STA $01		;  | set cape clipping y position
	LDA $13EC	;  | 
	STA $09		; / 
	LDA #$10	; \ set cape clipping height
	STA $03		; /
	JSL $03B69F	; get sprite clipping
	JSL $03B72B	; check for contact
	BCC .no_contact	; 
	JSR DamageSprite
	STZ $157C,x	; vertical direction = up
	LDA #$20	; \ disable contact
	STA $1FE2,x	; /
.no_contact
	RTS

FireballContact:
	LDA $7FAB28,x	; \ 
	AND #$20	;  | 
	BEQ .no_contact	; /
	LDA $166E,x	; \
	AND #$10	;  | 
	BNE .no_contact	; /
	LDY #$09	; times to loop
-	LDA $170B,y	; \
	CMP #$05	;  | 
	BEQ +		;  | only detect yoshi and mario fireballs
	CMP #$11	;  |
	BNE .loop	; /
+	LDA $171F,y	; \
	SEC		;  | 
	SBC #$02	;  | 
	STA $00		;  | fireball clipping x position
	LDA $1733,y	;  | 
	SBC #$00	;  | 
	STA $08		; /
	LDA #$0C	; \ fireball clipping width
	STA $02		; /
	LDA $1715,y	; \
	SEC		;  | 
	SBC #$04	;  | 
	STA $01		;  | fireball clipping y position
	LDA $1729,y	;  | 
	SBC #$00	;  | 
	STA $09		; /
	LDA #$0C	; \ fireball clipping height
	STA $03		; /
	JSL $03B69F	; get sprite clipping
	JSL $03B72B	; check for contact
	BCS +		; 
.loop	DEY		; 
	BPL -		; 
	RTS		; 
+	LDA #$01	; \
	STA $170B,y	;  | turn fireball to smoke
	LDA #$10	;  | 
	STA $176F,y	; /
	LDA $7FAB28,x	; \
	AND #$10	;  | branch if it shouldn't be damaged to fire
	BEQ .no_contact	; /
	STZ $157C,x	; vertical direction = up
	LDA #$0C	; \ timer to stop interaction
	STA $166E,x	; /
	JSR DamageSprite
.no_contact
	RTS

MarioContact:
	JSL $03B69F	; get sprite clipping
	LDA $1570,x	; 
	TAY		; 
	LDA $D8,x		; \
	CLC			;  | 
	ADC .interact_offset1,y	;  | 
	STA $05			;  | 
	LDA $14D4,x		;  | "tweak" sprite clipping
	ADC .interact_offset2,y	;  | 
	STA $0B			;  | 
	LDA .interact_height,y	;  | 
	STA $07			; /

	JSL $03B664	; get mario clipping
	JSL $03B72B	; check for contact
	BCC .no_contact	; 

	LDA $03		; \
	STA $00		;  | set a pseudo 16-bit value for mario's height clipping
	STZ $01		; /

	LDY $1570,x		; \
	LDA .interact_offset,y	;  | set a pseudo 16-bit offset for interaction
	STA $02			;  | 
	STZ $03			; /

	LDA $14D4,x	; \
	XBA		;  | 
	LDA $D8,x	;  | 
	REP #$20	;  | 
	LDY $187A	;  | 
	BEQ .noyoshi	;  | 
	SEC		;  | 
	SBC #$0010	;  | 
.noyoshi
	SEC		;  | 
	SBC $96		;  | branch if hitting sprite from below
	STA $04		;  | 
	CLC		;  | 
	ADC $00		;  | 
	CMP #$0018	; \
	BPL +		; /
.hitbottom2
	JMP .hitbottom	; 
+	LDA $04		; \
	CLC		;  | 
	ADC $02		;  | branch of above sprite
	CMP #$001E	;  | 
	SEP #$20	;  | 
	BPL .ontop	; /
.wall	STZ $7B		; no x speed
	JSR SubHorzPos	; \
	TYA		;  | index by side of sprite
	ASL		;  | 
	TAY		; /
	REP #$20	; \
	LDA $94		;  | 
	CLC		;  | push mario away from sprite
	ADC .mario_xoffset,y
	STA $94		;  | 
	SEP #$20	; /
.no_contact
	RTS		; 
.mario_xoffset
	dw $0001,$FFFF
.interact_offset
	db $00,$02,$04
.interact_offset1
	db $FA,$FC,$FE
.interact_offset2
	db $FF,$FF,$FF
.interact_height
	db $14,$12,$10
.ontop
	LDA $7D			; \ branch if going up
	BMI .no_contact	; / 
	LDA $7FAB28,x	; \
	AND #$80	;  | branch if can't be carried
	BEQ +		; /
	BIT $16		; \ don't carry if not pressing button
	BVC +		; /
	LDA $1470	; \
	ORA $187A	;  | don't carry if carrying already or riding yoshi
	BNE +		; /
	LDA #$0B	; \ set carried state
	STA $14C8,x	; /
+	LDA $72			; \ use normal solidity if on ground
	BEQ .notspinning	; /
	LDA $19			; \ use normal solidity if small
	BEQ .notspinning	; /
	LDA $1540,x		; \
	BNE .notspinning	; / use normal solidity if ability to spinjump disabled
	LDA $7FAB28,x		; \
	AND #$10		;  | branch if bit not set to allow spinjumping on sprite
	BEQ .notspinning	; /
	LDA $140D		; \
	BEQ .notspinning	; /
	STA $157C,x		; vertical direction = up
	LDA #$1C		; \
	STA $1540,x		; / set timer
	JSR DamageSprite
	LDA #$E0	; \ set y speed
	STA $7D		; /
.no_contact2
	RTS
.notspinning
	LDA #$01	; \ set "standing on surface" flag
	STA $1471	; /
	LDA #$10	; \ set y speed
	STA $7D		; /
	LDA $14D4,x	; \
	XBA		;  | 
	LDA $D8,x	;  | 
	REP #$20	;  | 
	CLC		;  | force mario above sprite
	ADC #$FFE0	;  | 
	LDY $187A	;  | \
	BEQ +		;  |  | factor in yoshi
	ADC #$FFEE	;  | /
+	ADC $02		;  | 
	STA $96		;  | 
	SEP #$20	; /
	RTS		; 
.hitbottom
	SEP #$20	; 
	LDA $7D		; \ branch if going down
	BPL .no_contact2 ;/
	STZ $7D		; nullify speed
	STZ $157C,x	; vertical direction = up
DamageSprite:
	LDA #$01	; \ play sound
	STA $1DF9	; /
	LDA $163E,x	; \ branch if timer set
	BEQ +		;  | 
	RTS		; /
+	LDA #$09	; \ set timer
	STA $163E,x	; /
DamageSprites:
	LDA #$18	; \ quake timer
	STA $1887	; /
	LDA $77		; \
	AND #$04	;  | branch if mario is in the air
	BEQ +		; /
	LDA #$B0	; \ set mario speed
	STA $7D		; /
+	LDY #$0B	; load times to loop
-	STX $00		; store current sprite index
	PHX		; 
	TYX		; 
	CPX $00		; \ if sprite being processed is this one, skip it
	BEQ .loop	; /
	LDA $1588,x	; \
	AND #$04	;  | skip all sprites in the air
	BEQ .loop	; /
	LDA $14C8,x	; \
	CMP #$08	;  | skip all killed sprites
	BCC .loop	; /
	PHY		; 
	JSR .damagesprite
	PLY		; 
.loop	PLX		; 
	DEY		; \ loop
	BPL -		; / 
	LDA #$09	; \ play sound
	STA $1DFC	; / 
	RTS		; 
.damagesprite
	LDY #$02	; load types of sprites to loop from whitelist
-	PHY		; 
	JSR +		; 
	PLY		; 
	DEY		; \ loop
	BPL -		; /
	RTS		; 
+	TYA		; \
	STA $00		;  | 
	ASL		;  | create index for Y
	CLC		;  | 
	ADC $00		;  | 
	TAY		; /
	LDA SpriteTable+$1,y	; \ branch if sprite is stated to be normal
	BEQ .normalsprite	; /
	LDA $7FAB10,x		; \
	AND #$08		;  | deal default damage if sprite checked doesn't match this criteria
	BEQ .defaultdamage	; /
	LDA $7FAB9E,x		; \
	CMP SpriteTable,y	;  | deal default damage if sprite checked does not match
	BNE .defaultdamage	; /
	BRA +			; branch to pointer
.normalsprite
	LDA $9E,x		; \
	CMP SpriteTable,y	;  | branch if current sprite matches whitelisted sprite
	BEQ +			; /
.defaultdamage
	LDA #$00		; \ set default damage type
	BRA ++			; /
+	LDA SpriteTable+$2,y	; \ load sprite damage type table
++	JSL $0086DF		; /
	dw DefaultDamage	; $00	= references the original "cape smash"/"hit sprite with block" routine.
	dw DamageType1		; $01	= standard Mario Bros enemy; Spiny, Side Stepper, and Fighter Fly.
;; BOOKMARK1; add more damage type pointer entries here; actual code each entry points to should be placed below.

;; default damage; reference original code
DefaultDamage:
	LDA $166E,x		; \ don't stun if it is immune.
	AND #$20		; /
	ORA $15D0,x		; don't stun if being eaten
	ORA $154C,x		; don't stun if not interacting with mario
	BNE .return		; 
	PHK			; \
	PEA.w .sprsmash-1	;  | JSL2RTS; uses the original code.
	PEA.w $02B889-1		;  | 
	JML $029404		; /
.sprsmash
	LDY #$08		; \
	LDA $157C,x		;  | 
	BEQ +			;  | set x speed based on direction
	LDY #$F8		;  | 
+	STY $B6,x		; /
.return	RTS			; 

;; Mario Bros-type enemy damage; mark sprite as being damaged
DamageType1:
	LDA $1FD6,x		; \
	ORA #$80		;  | set "damaged" flag.
	STA $1FD6,x		; / the "Mario Bros" sprites will know what to make of this, and take damage.
	LDA #$E0		; \ set y speed
	STA $AA,x		; /
	RTS			; 

SubGfx:
	JSR GET_DRAW_INFO

	LDA #$F0	; \ hide oam tile behind sprite when carriable
	STA $0309,y	; /

	LDA $163E,x	; \
	PHX		;  | 
	TAX		;  | set offset based on timer
	LDA .yoffset2,x	;  | 
	STA $03		; /
	PLX		; 

	LDA $157C,x	; \
	STA $04		; /

	STZ $05
	LDA $1510,x
	BEQ +
	LDA $151C,x
	STA $05
+
	LDA $151C,x	; \
	CMP #$03	;  | 
	BCC +		;  | manage frame information
	LDA #$02	;  | 
+	STA $02		; /
	LSR		; \
	PHX		;  | use above stuff to load tiles to draw
	TAX		; /
-	LDA $00		; \
	CLC		;  | x offset
	ADC .xoffset,x	;  | 
	STA $0300,y	; /

	PHX		; 
	LDA $02		; \  \ index base y offset by state
	TAX		;  | / 
	LDA $03		;  | \ load "animated" y offset
	PHX		;  |  | 
	LDX $04		;  |  | \
	BEQ +		;  |  |  | flip it around if it should push downwards
	EOR #$FF	;  |  |  | 
	INC		;  |  | /
+	PLX		;  | /
	CLC		;  | setup y offset
	ADC .yoffset,x	;  | 
	ADC $01		;  | 
	LDX $05		;  | 
	ADC .yoffset3,x	;  | 
	DEC		;  | 
	STA $0301,y	; /
	PLX		; 

	LDA #$06	; \
	ORA #$01	;  | 
	ORA $64		;  | 
	STA $0303,y	; /

	PHX		; \
	TXA		;  | 
	CLC		;  | create index for tilemap
	ADC $02		;  | 
	TAX		; /

	LDA .tilemap,x	; \
	STA $0302,y	; /

	PHY		; 
	TYA		; 
	LSR : LSR	; 
	TAY		; 
	LDA $02		; \
	EOR #$02	;  | 
	LSR		;  | set up tilesize
	ASL		;  | 
	STA $0460,y	; /
	PLY		; 
	PLX		; 

	INY : INY : INY : INY
	DEX
	BPL -
	PLX

	LDY #$FF	; 
	LDA $151C,x	; \
	CMP #$03	;  | 
	BCC +		;  | TODO: find out why i did this
	LDA #$00	;  | 
+	LSR		; /
	JSL $01B7B3	; 
	RTS
.tilemap
	db $88,$8A	; 1 16x16
	db $8C,$8D	; 2 8x8

.tilesize
	db $02,$02,$00

.xoffset
	db $00,$08

.yoffset
	db $00,$00,$04

.yoffset2
	db $00,$FE,$FC,$FA,$F9,$F8,$F9,$FA,$FC,$FE

.yoffset3
	db $00,$02,$04

SubSmoke:
	LDA #$01	; \ load times to run code
	STA $00		; /
--	LDY #$07	; load times to loop
-	LDA $170B,y	; \ branch if not free slot
	BNE +		; /
	JSR .spawnsmoke	; spawn smoke
	BRA ++		; done searching
+	DEY		; \ loop if not a free slot
	BPL -		; /
++	DEC $00		; \
	LDA $00		;  | loop once more
	BPL --		; /
	RTS		; 
.spawnsmoke
	LDA #$0F	; \ extended sprite number
	STA $170B,y	; /

	LDA $14D4,x	; \
	XBA		;  | 
	LDA $D8,x	;  | 
	REP #$20	;  | 
	CLC		;  | y position from sprite
	ADC #$0008	;  | 
	SEP #$20	;  | 
	STA $1715,y	;  | 
	XBA		;  | 
	STA $1729,y	; /


	LDA $E4,x	; \
	STA $171F,y	;  | x position from sprite
	LDA $14E0,x	;  | 
	STA $1733,y	; /

	PHX		; \
	LDX $00		;  | 
	LDA .xspd,x	;  | set x speed
	PLX		;  | 
	STA $1747,y	; /
	LDA #$10	; \ set timer to live
	STA $176F,y	; /
	RTS		; 
.xspd	db $18,$E8
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ROUTINES FROM THE LIBRARY ARE PASTED BELOW
; You should never have to modify this code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;       Y = index to sprite OAM ($300)
;       $00 = sprite x position relative to screen boarder
;       $01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1:             db $0C,$1C
SPR_T2:             db $01,$02

GET_DRAW_INFO:      STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X:        LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.w #$0040            ;  |
                    CMP.w #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP:     LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y:        DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID:            PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12:            db $40,$B0
SPR_T13:            db $01,$FF
SPR_T14:            db $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
		            db $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15:            db $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
					db $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1:  LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2:  LDA #$04                ;  | 
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3:  LDA #$06                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4:  LDA #$08                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5:  LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6:  LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7:  LDA #$0E                ;  |
STORE_03:  STA $03					;  |            
					BRA START_SUB			;  |
SUB_OFF_SCREEN_X0:  STZ $03					; /

START_SUB:          JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ; | 
                    ADC #$30                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; | 
                    CMP #$02                ; | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31:            LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE:       LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE:        STZ $14C8,x             ; erase sprite
RETURN_35:          RTS                     ; return

VERTICAL_LEVEL:     LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.w $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38:            LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN:  LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ; |  
                    RTS                     ; / return

SubHorzPos:

	LDY #$00	; 
	LDA $14E0,x	; \
	XBA		;  | 
	LDA $E4,x	;  | 
	REP #$20	;  | get subtract sprite's x position by mario's
	SEC		;  | 
	SBC $94		;  | 
	BMI +		;  | 
	INY		;    increment y if positive 
+	STA $0C		;  | 
	SEP #$20	; /
	RTS		; 


SubVertPos:
	LDY #$00	; 
	LDA $14D4,x	; 
	XBA		;  | 
	LDA $D8,x	;  | 
	REP #$20	;  | get subtract sprite's y position by mario's
	SEC		;  | 
	SBC $96		;  | 
	BMI +		;  | 
	INY		;    increment y if positive
+	STA $0E		;  | 
	SEP #$20	; /
	RTS		; 